# -*- coding: utf-8 -*-
#
#   OghmaNano - Organic and hybrid Material Nano Simulation tool
#   Copyright (C) 2008-2022 Roderick C. I. MacKenzie r.c.i.mackenzie at googlemail.com
#
#   https://www.oghma-nano.com
#
#   Permission is hereby granted, free of charge, to any person obtaining a
#   copy of this software and associated documentation files (the "Software"),
#   to deal in the Software without restriction, including without limitation
#   the rights to use, copy, modify, merge, publish, distribute, sublicense, 
#   and/or sell copies of the Software, and to permit persons to whom the
#   Software is furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included
#   in all copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#   THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
#   SOFTWARE.
#

## @package optics_ribbon
#  The ribbon for the optics window
#



#qt
from PySide2.QtWidgets import QMainWindow, QTextEdit, QAction, QApplication
from PySide2.QtGui import QIcon, QTextFormat,QTextOption,QTextCursor
from gQtCore import QSize, Qt,QFile,QIODevice,QRect
from PySide2.QtWidgets import QWidget,QSizePolicy, QPlainTextEdit,QVBoxLayout,QHBoxLayout, QPushButton,QDialog,QFileDialog,QToolBar, QMessageBox, QLineEdit, QToolButton
from PySide2.QtWidgets import QTabWidget

from PySide2.QtGui import QPainter,QColor
from icon_lib import icon_get

from util import wrap_text
from ribbon_base import ribbon_base
from play import play
from QAction_lock import QAction_lock
from cal_path import sim_paths

from gQtCore import QFile, QRegExp, Qt
from PySide2.QtGui import QFont, QSyntaxHighlighter, QTextCharFormat

class QLineNumberArea(QWidget):
	def __init__(self, editor):
		super().__init__(editor)
		self.codeEditor = editor

	def sizeHint(self):
		return QSize(self.editor.lineNumberAreaWidth(), 0)

	def paintEvent(self, event):
		self.codeEditor.lineNumberAreaPaintEvent(event)


class code_editor(QPlainTextEdit):
	def __init__(self, parent=None):
		super().__init__(parent)
		self.setWordWrapMode(QTextOption.NoWrap)
		self.lineNumberArea = QLineNumberArea(self)
		self.blockCountChanged.connect(self.updateLineNumberAreaWidth)
		self.updateRequest.connect(self.updateLineNumberArea)
		self.cursorPositionChanged.connect(self.highlightCurrentLine)
		self.updateLineNumberAreaWidth(0)
		#self.setTabStopDistance(self.fontMetrics().horizontalAdvance(' ') * 4)  # Set tab width

	def keyPressEvent(self, event):
		cursor = self.textCursor()
		selected_text = cursor.selection().toPlainText()

		if event.key() == Qt.Key_Tab and not event.modifiers():
			if selected_text:  # If there is selected text, indent all selected lines
				self.indent_selection()
			else:
				cursor.insertText("\t")
		elif event.key() == Qt.Key_Backtab:  # Shift+Tab
			self.unindent_selection()
		else:
			super().keyPressEvent(event)

	def indent_selection(self):
		cursor = self.textCursor()
		start = cursor.selectionStart()
		end = cursor.selectionEnd()
		cursor.beginEditBlock()
		cursor.setPosition(start)

		while cursor.position() <= end:
			cursor.movePosition(QTextCursor.StartOfBlock)
			cursor.insertText("\t")
			cursor.movePosition(QTextCursor.NextBlock)
			if cursor.position() > end:
				break

		cursor.endEditBlock()


	def unindent_selection(self):
		cursor = self.textCursor()
		start = cursor.selectionStart()
		end = cursor.selectionEnd()
		cursor.beginEditBlock()

		cursor.setPosition(start)
		while cursor.position() <= end:
			cursor.movePosition(QTextCursor.StartOfBlock)
			
			# Select first 1 or 4 characters
			cursor.movePosition(QTextCursor.Right, QTextCursor.KeepAnchor, 1)
			text = cursor.selectedText()
			if text == "\t":
				cursor.removeSelectedText()
			else:
				cursor.movePosition(QTextCursor.StartOfBlock)
				cursor.movePosition(QTextCursor.Right, QTextCursor.KeepAnchor, 4)
				text = cursor.selectedText()
				if text == "    ":
					cursor.removeSelectedText()

			cursor.movePosition(QTextCursor.NextBlock)
			if cursor.position() > end:
				break

		cursor.endEditBlock()

	def lineNumberAreaWidth(self):
		digits = 1
		max_value = max(1, self.blockCount())
		while max_value >= 10:
			max_value /= 10
			digits += 1
		space = 3 + self.fontMetrics().width('9') * digits
		return space

	def updateLineNumberAreaWidth(self, _):
		self.setViewportMargins(self.lineNumberAreaWidth(), 0, 0, 0)

	def updateLineNumberArea(self, rect, dy):
		if dy:
			self.lineNumberArea.scroll(0, dy)
		else:
			self.lineNumberArea.update(0, rect.y(), self.lineNumberArea.width(), rect.height())
		if rect.contains(self.viewport().rect()):
			self.updateLineNumberAreaWidth(0)

	def resizeEvent(self, event):
		super().resizeEvent(event)
		cr = self.contentsRect()
		self.lineNumberArea.setGeometry(QRect(cr.left(), cr.top(), self.lineNumberAreaWidth(), cr.height()))

	def highlightCurrentLine(self):
		extraSelections = []
		if not self.isReadOnly():
			selection = QTextEdit.ExtraSelection()
			lineColor = QColor(Qt.yellow).lighter(160)
			selection.format.setBackground(lineColor)
			selection.format.setProperty(QTextFormat.FullWidthSelection, True)
			selection.cursor = self.textCursor()
			selection.cursor.clearSelection()
			extraSelections.append(selection)
		self.setExtraSelections(extraSelections)

	def lineNumberAreaPaintEvent(self, event):
		painter = QPainter(self.lineNumberArea)

		painter.fillRect(event.rect(), Qt.lightGray)

		block = self.firstVisibleBlock()
		blockNumber = block.blockNumber()
		top = self.blockBoundingGeometry(block).translated(self.contentOffset()).top()
		bottom = top + self.blockBoundingRect(block).height()

		# Just to make sure I use the right font
		height = self.fontMetrics().height()
		while block.isValid() and (top <= event.rect().bottom()):
			if block.isVisible() and (bottom >= event.rect().top()):
				number = str(blockNumber + 1)
				painter.setPen(Qt.black)
				painter.drawText(0, top, self.lineNumberArea.width(), height, Qt.AlignRight, number)

			block = block.next()
			top = bottom
			bottom = top + self.blockBoundingRect(block).height()
			blockNumber += 1

